const path = require('path')
const webpack = require('webpack')
const HappyPack = require('happypack')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const CopyWebpackPlugin = require('copy-webpack-plugin')
const { projectPath, resolve } = require('../utils')
const appConfig = require('../utils/get-app-config')
const vueOptions = require('./vue-loader.conf')
const styleOptions = require('./style-loader')

const config = {
  entry: appConfig.webpack.entry,
  mode: 'none',
  devtool: '#eval-source-map',
  output: {
    filename: '[name].js',
    path: `${projectPath}/public`,
  },
  module: {
    rules: [
      {
        test: /\.jsx?$/,
        loader: 'happypack/loader?id=babel',
        include: resolve('client'),
      },
      {
        test: /\.vue$/,
        loader: 'vue-loader',
        include: resolve('client'),
        options: vueOptions,
      },
      {
        test: /\.css$/,
        loader: styleOptions.cssLoader({}).css,
        include: [
          resolve('client'),
          resolve('node_modules/normalize.css'),
          // resolve('node_modules/element-ui/lib/theme-chalk/index.css'),
        ],
      },
      {
        test: /\.styl$/,
        loader: styleOptions.cssLoader({}).stylus,
        include: [resolve('client'), resolve('node_modules/normalize.css')],
      },
      {
        test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
        loaders: [
          {
            loader: 'url-loader',
            options: {
              limit: 8192,
              name: '[name].[ext]?[hash]',
            },
          },
        ],
      },
      {
        test: /\.(woff2?|ttf|eot|otf)$/,
        loaders: [
          {
            loader: 'file-loader',
            options: {
              name: 'fonts/[name].[ext]?[hash]',
            },
          },
        ],
      }
    ],
  },
  resolve: {
    alias: {
      'vue$': 'vue/dist/vue.esm.js',
      '@': resolve('client'),
    }
  },
  plugins: [
    new HtmlWebpackPlugin({
      filename: 'index.html',
      template: path.join(projectPath, 'index.template.ejs'),
      inject: true,
    }),
    new webpack.DllReferencePlugin({
      manifest: require(`${projectPath}/vendor_manifest.json`)
    }),
    new CopyWebpackPlugin([
      {
        from: `${projectPath}/static`,
        to: 'static',
      },
    ]),
    new HappyPack({
      id: 'babel',
      threads: 4,
      loaders: ['babel-loader?cacheDirectory=true'],
    }),
  ],
}

if (appConfig.webpack.rules instanceof Array && appConfig.webpack.rules.length) {
  config.module.rules.push(...appConfig.webpack.rules)
}

module.exports = config
